/*-----------------------------------------------------------------------------
 * File: R9346.c
 * Autor: Jochen Beck - DH1SAJ
 *
 * Function: Access to serial EEPROM 93C46 typically fitted on a 
 *           RTL8019AS ISA-BUS NIC.
 *           
-----------------------------------------------------------------------------*/

#include <iom128v.h> 
#include <stdio.h>
#include <macros.h>

//#include <uartout.h>
#include <8019as.h>

volatile char *RTLCR  = (char *)0x8300;
volatile char *NVM    = (char *)0x8300+_9346CR;


//*******************************************************************************
// AT_BASE
//
// description: sets the base address for all upcoming 93C46 access
//*******************************************************************************
void AT_BASE(volatile char *base)
{	
	RTLCR  = base;
    NVM    = base+_9346CR;
}


//*******************************************************************************
// AT_INIT
//
// description: prepares the NIC for accessing the on board EEPROM
//*******************************************************************************
void AT_INIT(void)
{char value;
	
	*(RTLCR) = 0xE1;	// change to page 3
	*(NVM)   = BASE;    // 9346 programming mode
}


//*******************************************************************************
// AT_SEND_CMD
//
// description: send an instruction to the on board EEPROM
//*******************************************************************************
void AT_SEND_CMD(int data, char close)
{
 char loop, value;

	AT_INIT();

	*(NVM) = BASE + CS;          // NVM Chip Select
    data |= BIT(8);                     // Starting Bit always '1'
	
	for (loop=0;loop<9;loop++)
     {
      if (data & BIT(8)) 
	    value = BASE + CS + DI;
	   else
	    value = BASE + CS;
	  *(NVM) = value;     
      *(NVM) = value + SK;     
      data <<= 1;
	  *(NVM) = value;     
     }	  

	if (close) *(NVM) = BASE;    // Chip unselect     
}


//*******************************************************************************
// AT_EWEN
//
// description: enables writing/erasing the on board EEPROM
//*******************************************************************************
void AT_EWEN(void)
{
    AT_SEND_CMD(0x30,1);
}


//*******************************************************************************
// AT_EWDS
//
// description: disables writing/erasing the on board EEPROM
//*******************************************************************************
void AT_EWDS(void)
{
    AT_SEND_CMD(0x00,1);	
}
	
//*******************************************************************************
// AT_READ
//
// description: does a read at the given address of the on board EEPROM
//*******************************************************************************
unsigned int AT_READ(char adr)
{
 unsigned int data = 0;
 char loop, value;
	
	adr &= 0x3F;    // Adr consists of 6 Bit only
    adr |= 0x80;    // Opcode for READ  '10' @ Bit 6&7
    AT_SEND_CMD(adr,0);  // Send read-address, keep NVM open

	for (loop=0;loop<16;loop++)
     {
	  *(NVM) = BASE + CS + SK;
      data <<= 1;
	  value = (*(NVM) & DO);    // Dummy readin for wasting time     
	  value = *(NVM);           // Dummy readin for wasting time     
	  value = (*(NVM) & DO);    // HOT readin of DO pin 
	  *(NVM) = BASE + CS;
      if (value) data |= BIT(0);
	 }	  
	
	*(NVM) = BASE;    // Chip unselect     
	return (data);
}

//*******************************************************************************
// AT_WRITE
//
// description: does a write to the given address of the on board EEPROM
//*******************************************************************************
void AT_WRITE(char adr, unsigned int data)
{
 char loop, value;
	
    AT_EWEN();           // NVM write enable 
	adr &= 0x3F;         // Adr consists of 6 Bit only
    adr |= 0x40;         // Opcode for WRITE  '01' @ Bit 6&7
    AT_SEND_CMD(adr,0);  // Send write-address, keep NVM open

	for (loop=0;loop<16;loop++)
     {
      if (data & BIT(15)) 
	    value = BASE + CS + DI;
	   else
	    value = BASE + CS;
	  *(NVM) = value;     
      *(NVM) = value + SK;     
      data <<= 1;
	  *(NVM) = value;     
     }	  
	*(NVM) = BASE;    // Chip unselect     
    AT_EWDS();               // NVM write disable
}

//*******************************************************************************
// AT_ERASE
//
// description: erases Byte at given adr
//*******************************************************************************
void AT_ERASE(char adr)
{
    AT_EWEN();           // NVM write enable 
	adr &= 0x3F;         // Adr consists of 6 Bit only
    adr |= 0xC0;         // Opcode for READ  '10' @ Bit 6&7
    AT_SEND_CMD(adr,1);  // Send erase-address, close NVM
    AT_EWDS();           // NVM write disable
}

//*******************************************************************************
// AT_ERALL
//
// description: erases all Bytes in NVM
//*******************************************************************************
void AT_ERALL(void)
{
    AT_EWEN();            // NVM write enable 
    AT_SEND_CMD(0x20,1);  // Send eglobal erase cmd, close NVM
    AT_EWDS();            // NVM write disable
}

//*******************************************************************************
// AT_CHECK
//
// description: Check for chip ready
//*******************************************************************************
char AT_CHECK(void)
{ 
 char value;

    AT_INIT(); 
	*(NVM) = BASE + CS;
    NOP(); NOP();
    value = (*(NVM) & DI);     
	*(NVM) = BASE;

    return (value);
}

//*******************************************************************************
// AT_RELOAD
//
// description: reload 93C46 Config1-4 to RTL8019
//*******************************************************************************
void AT_RELOAD(void)
{unsigned int wait; 

	*(RTLCR) = 0xE1;	// change to page 3
	*(NVM)   = 0x40;    // 9346 programming mode
    for (wait=0; wait < 40000; wait++) NOP();
}


