/*-----------------------------------------------------------------------------
 * File: ISANUT.C
 * Autor: Jochen Beck - DH1SAJ
 *
 *
 ** Portions of code taken from EGNITEs basemon.c sample for ETHERNUT **
 *
 *
 * Function: Detects and configures a ISA NIC with RTL8019AS 
 *           for usage with EtherNut.
 *
 * Version: 0.0.1
 *
 *  Target: ATMega128-16 (NOT in ATMega103 compatibility mode)
 *          ATMega128L-8 is OK if running @5V, even with fclk > 8 MHz !
 *  XTAL: 14.7456 MHz
 *
 * LOCK BITS: All unprogrammed "1" --> 0xFF
 *            For ICCAVR: 0x0000
 * FUSE BITS:
   M103C    unprogged "1"
   WDTON    unprogged "1"

   OCDEN    unprogged "1"
   JTAGEN   unprogged "1"
   SPIEN    progged   "0"
   CKOPT    progged   "0"
   EESAVE   unprogged "1"
   BOOTSZ1  progged   "0"
   BOOTSZ0  progged   "0"
   BOOTRST  unprogged "1"
   
   BODLEVEL unprogged "1"    (Ub = 5V)
   BODEN    progged   "0"    (Brown Out Detect ON)
   SUT1     unprogged "1"    
   SUT0     progged   "0"    (Start Up slow)
   CKSEL3   unprogged "1"    (External XTAL > 1MHz)
   CKSEL2   unprogged "1"
   CKSEL1   unprogged "1"
   CKSEL0   unprogged "1"
    --> 0xFFC9AF
	For ICCAVR: 0x003650
 * --------------------------------------------------------------------------*/

#include <iom128v.h> 
#include <macros.h>
#include <stdio.h>

#include <uartout.h>
#include <8019as.h>
#include <R9346.h>


#define VERSION   1            // Laufende Versionsnummer



unsigned int sram;
volatile char *RTLbase = (char *)0x8300;
unsigned char help;
unsigned char flagbyte;


/* *** Code der Interruptfunktionen *** */


#pragma interrupt_handler timer0_ovf_isr:17
void timer0_ovf_isr(void)
{
 TCNT0 = 0x80; //reload counter value
 flagbyte |= BIT(0);
}

#pragma interrupt_handler int5_isr:7
void int5_isr(void)
{
 //external interupt on INT5
}


/* *** Code of regular funktions *** */

//TIMER0 initialisation - prescale:256
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 ASSR  = 0x08; //set async mode
 TCNT0 = 0x80; //set count = 1sec @ 32768Hz
 TCCR0 = 0x06; //start timer
}

void port_init(void)
{
 DDRA  = 0x00;
 DDRC  = 0x00;

 PORTB = 0xFF;
 DDRB  = 0x00;
 PORTD = 0xFF;
 DDRD  = 0xFF;
 PORTE = 0x00;
 DDRE  = 0x00;
 PORTF = 0x00;
 DDRF  = 0x00;
}


void init_devices(void)
{
 CLI(); //disable all interrupts
 XDIV  = 0x00; //xtal divider
 XMCRA = 0x00; //external memory
 XMCRB = 0x80; //external memory
 port_init();

 InitUART0(95); // 95 = 9k6 @ 14.7456MHz
 
 MCUCR = 0x80;
 EICRA = 0x00; //extended ext ints
 EICRB = 0x08; //IRQ5 triggers on falling edge
 EIMSK = 0x00; //0x20 to activate IRQ5
 TIMSK = 0x01;  //timer interrupt sources
 ETIMSK = 0x00; //extended timer interrupt sources
 SEI(); //re-enable interrupts
}

/*
 * Delay by executing a given number of NOPs.
 */
void Delay(long nops)
{
    volatile long i;
    for(i = 0; i < nops; i++) NOP();
}

/*
 * Test external SRAM.
 */
unsigned int TestExternalRam(unsigned int first, unsigned int last)
{
    volatile char *mem;
    unsigned int all_low, all_high;
    unsigned int short_high, short_low;
    char pat8;
    unsigned int pat16;
    unsigned int ipat16;
    unsigned int msk16;


    /*
     * Check data bus integrity.
     * -------------------------
     */
    all_low = 0;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        *mem = 0xFF;
        if(*mem != 0xFF) {
            if(all_low)
                all_low &= ~*mem;
            else
                all_low = ~*mem;
        }
        if(all_low == 0)
            break;
    }
    if(all_low)
        printf("Data bus bits %02X always low\r\n\x07", all_low);

    all_high = 0;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        *mem = 0;
        if(*mem) {
            if(all_high)
                all_high &= *mem;
            else
                all_high = *mem;
        }
        if(all_high == 0)
            break;
    }
    if(all_high)
        printf("Data bus bits %02X always high\r\n\x07", all_high);

    short_high = 0;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(short_high && (short_high & (unsigned int)mem) != 0 && 
		  (short_high & (unsigned int)mem) != short_high)
            continue;
        for(pat8 = 1; pat8; pat8 <<= 1) {
            *mem = pat8;
            if((*mem & ~pat8) != 0) {
                if(short_high)
                    short_high &= *mem;
                else
                    short_high = *mem;
            }
        }
        if(short_high == 0) 
            break;
    }

    short_low = short_high;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(short_low && (short_low & (unsigned int)mem) != 0 && (short_low & (unsigned int)mem) != short_low)
            continue;
        for(pat8 = 1; pat8; pat8 <<= 1) {
            *mem = ~pat8;
            if((*mem | pat8) != 0xFF) {
                if(short_low)
                    short_low &= ~*mem;
                else
                    short_low = ~*mem & 0xFF;
            }
        }
        if(short_low == 0)
            break;
    }

    if(short_high | short_low)
        printf("Data bus bits 0x%02X may be shortened\r\n\x07", short_high | short_low);
    if(short_high || short_low || all_high || all_low) 
        return 0;

    /*
     * Check address bus integrity.
     * ----------------------------
     */
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1)
        *mem = 0;
    all_low = 0;
    for(msk16 = 0xFFFF; msk16 > last; msk16 >>= 1);
    *((char *)msk16) = 0xAA;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(*mem == 0xAA && (unsigned int)mem != msk16) {
            if(all_low)
                all_low &= ~(unsigned int)mem;
            else
                all_low = ~(unsigned int)mem & msk16;
        }
    }
    *((char *)msk16) = 0x55;
    for(mem = (char *)first; (unsigned int)mem < last; mem+=1) {
        if(*mem == 0x55 && (unsigned int)mem != msk16) {
            if(all_low)
                all_low &= ~(unsigned int)mem;
            else
                all_low = ~(unsigned int)mem & msk16;
        }
    }
    *((char *)msk16) = 0;
    if(all_low)
        printf("Address bus bits %04X always low\r\n\x07", all_low);

    all_high = 0;
    for(pat16 = 0x8000; pat16 > last; pat16 >>= 1);
    *((char *)pat16) = 0xAA;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(*mem == 0xAA && (unsigned int)mem != pat16) {
            if(all_high)
                all_high &= (unsigned int)mem;
            else
                all_high = (unsigned int)mem;
        }
    }
    *((char *)pat16) = 0x55;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(*mem == 0x55 && (unsigned int)mem != pat16) {
            if(all_high)
                all_high &= (unsigned int)mem;
            else
                all_high = (unsigned int)mem;
        }
    }
    *((char *)pat16) = 0;

    for(pat16 = 1; pat16 <= first; pat16 <<= 1);
    *((char *)pat16) = 0xAA;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(*mem == 0xAA && (unsigned int)mem != pat16) {
            if(all_high)
                all_high &= (unsigned int)mem;
            else
                all_high = (unsigned int)mem;
        }
    }
    *((char *)pat16) = 0x55;
    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        if(*mem == 0x55 && (unsigned int)mem != pat16) {
            if(all_high)
                all_high &= (unsigned int)mem;
            else
                all_high = (unsigned int)mem;
        }
    }
    *((char *)pat16) = 0;
    if(all_high)
        printf("Address bus bits %04X always high\r\n\x07", all_high);

    short_high = 0;
    for(pat16 = 0x0100; pat16 <= last; pat16 <<= 1) {
        if(pat16 < first)
            continue;
        *((char *)pat16) = 0x55;
        for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
            if(*mem == 0x55 && ((unsigned int)mem & ~pat16) != 0) {
                *((char *)pat16) = 0xAA;
                if(*mem == 0xAA) {
                    if(short_high) 
                        short_high &= (unsigned int)mem;
                    else 
                        short_high = (unsigned int)mem;
                }
                *((char *)pat16) = 0x55;
            }
        }
        *((char *)pat16) = 0;
    }

    short_low = 0;
    for(pat16 = 0x0100; pat16 <= last; pat16 <<= 1) {
        ipat16 = ~pat16 & msk16;
        if(ipat16 < first)
            continue;
        *((char *)ipat16) = 0x55;
        for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
            if(*mem == 0x55 && ((unsigned int)mem | pat16) != msk16) {
                *mem = 0xAA;
                if(*((char *)ipat16) == 0xAA) {
                    if(short_low) 
                        short_low &= ~((unsigned int)mem & ipat16);
                    else 
                        short_low = ~((unsigned int)mem & ipat16) & msk16;
                }
                *mem = 0x55;
            }
        }
        *((char *)ipat16) = 0;
    }

    if(short_high | short_low)
        printf("Address bus bits %04X may be shortened\r\n\x07", short_low | short_high);
    if(short_high || short_low || all_high || all_low)
        return 0;

    for(mem = (char *)first; (unsigned int)mem <= last; mem+=1) {
        for(pat8 = 1; pat8; pat8 <<= 1) {
            *mem = pat8;
            if(*mem != pat8) {
                last = (unsigned int)mem;
                printf( "Memory chip error at %04X\r\n\x07", last);
                break;
            }
        }
    }
    return last - first + 1;
}

unsigned int FindRealtek(void)
{
    volatile char *base = (char *)0x8300;
    int i;
    unsigned int ver_id, nic_id, first_id=0;

   for(base = (char *)0x8000; base <= (char *)0xffdf; base += 0x20) 
    {
      printf("NIC 0x%04X reset...", (unsigned int)base);
      *(base + 0x1f) = *(base + 0x1f);
      Delay(40000);
      for(i = 0; i < 255; i++) 
	   {
         if(*(base + 7) & 0x80)  break;
       }
      if(i < 255) 
	   {
         nic_id = *(base + 10) << 8;
         nic_id |= *(base + 11);
         printf("OK id=0x%04X", nic_id);
         if(nic_id != 0x5070)
            printf(" failed\r");
          else
           { printf(" OK  ");
		     if (!first_id) 
			   { first_id = (unsigned int)base;
				 *(base) = 0xE1;
				 ver_id = *(base + 3) & 0xC0;
			     printf("Chip-Rev.: RTL8019");
                 if (ver_id < 0xC0) putchar('A');
                 if (!ver_id) putchar('S');
                 putchar('\n');
			   }
		   }
       }
        else
          printf(" failed             \r");
    }
  printf("\r                                        \n");
  return first_id;
}


void NVMread(char first, char last)
{
  unsigned int val1;
  
  for (help=first;help<last;help++)        
   {                                       
	val1 = AT_READ(help);
    printf("EEPROM-Adr: 0x%02x  ", help);
	printf("Value: 0x%04x\n",val1);
   }
}

void NVMwrite(char ads, unsigned int word)
{
  while(AT_CHECK()) ;   // Wait until pervious word is physically stored in NVM
  AT_WRITE(ads,word);     
  printf("Written 0x%04X to 0x%02X\n",word,ads);
}

void main()  /***MAIN***/
{
 unsigned int  val1=0, b,c;
 //char a;
 
  flagbyte = 0;   
  init_devices();

  Delay(50000);               // Just wait for UART to initialise prescaler etc
  printf("\n\nBooting...\n\n");

  printf("Testing for 32.768kHz Xtal...");
  timer0_init();
  Delay(200000);               
  if (TCNT0 == 0) printf("missing");
   else if ((TCNT0 > 0xA0) && (TCNT0 < 0xF0)) printf("OK");
    else printf("\nCheck your XTAL frequencies !");
  
  c=0;
  for (b=0;b<100;b++)
   if (PINE & 0x20) c++;
  if (c<95) printf("\n\nNIC-IRQ (PE5) stuck LOW");
      
  printf("\n\nExternal SRAM Test...");
  if(sram = TestExternalRam(0x1100, 0x7FFF)) 
    printf(" %d (0x%04X) Bytes OK\n\n",sram,sram);

  do{ printf("Searching IO-Range for RTL8019\n");
      RTLbase = (char *)FindRealtek();
	}while(!RTLbase); 
    
   AT_BASE((char *)RTLbase);
  printf("Reading 93C46 contents\n");
  NVMread(0,5);

  printf("\nWriting configuration to 93C46\n");
  NVMwrite(0, 0x4080);       // Config 2 & 1
  NVMwrite(1, 0x0130);       // Config 4 & 3
//  NVMwrite(2, 0x0000);       // MAC Byte1 & Byte0
//  NVMwrite(3, 0x9800);       // MAC Byte3 & Byte2
//  NVMwrite(4, 0x0006);       // MAC Byte5 & Byte4

  printf("\nReading back 93C46 contents\n");
  NVMread(0,5);
  AT_RELOAD();

  printf("\nConfirm new RTL8019 mapping\n");
  if( (char *)FindRealtek() != (char *)0x8300) ;

   
  Delay(3000000);   // Just waste lots of time
  printf("Endurance test starting...\n");
  Delay(1000000);   // Just waste more time

  
            /*  ** Loop Forever, may be used as endurance test **  */
  do{
      printf("\n");

      printf("External SRAM Test...");
      if(sram = TestExternalRam(0x1100, 0x7FFF)) 
      printf(" %d (0x%04X) Bytes OK\n",sram,sram);

      printf("Searching IO-Range for RTL8019\n");
      RTLbase = (char *)FindRealtek(); 

      printf("External SRAM Test...");
      if(sram = TestExternalRam(0x1100, 0x7FFF)) 
      printf(" %d (0x%04X) Bytes OK\n",sram,sram);

	}while(1);

}

