/* Portions of code adapted from Atmel AVR Application Note AVR306
 * Interrupt mode driver for UART.
 */
#include <iom128v.h>
#include <macros.h>


/* IMPORTANT: the vector numbers are for Mega128! If you use other devices
 * you must change them to the different set of numbers.
 *
 * UART_RX_interrupt set to UART, Rx Complete
 * UART_TX_interrupt set to UART Data Register Empty
 */


/* UART Buffer Defines */
#define UART_RX_BUFFER_SIZE 16 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#define UART_TX_BUFFER_SIZE 256 /* 1,2,4,8,16,32,64,128 or 256 bytes */
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )

#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
#error RX buffer size is not a power of 2
#endif
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
#error TX buffer size is not a power of 2
#endif

/* Static Variables */
static unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
static volatile unsigned char UART_RxHead;
static volatile unsigned char UART_RxTail;
static unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
static volatile unsigned char UART_TxHead;
static volatile unsigned char UART_TxTail;

/* initialize UART */
void InitUART0( unsigned int baudrate )
{
	unsigned char x;
	UCSR0B = 0;
    UCSR0A = 0x00;
    UCSR0C = 0x06;
	UBRR0L = (unsigned char) baudrate; /* set the baud rate */
	UBRR0H = baudrate >> 8;            /* set the baud rate */
	x = 0; /* flush receive buffer */
	UART_RxTail = x;
	UART_RxHead = x;
	UART_TxTail = x;
	UART_TxHead = x;
	/* enable UART receiver and transmitter, and receive interrupt */
	UCSR0B = ( (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0) );
}

/* interrupt handlers */
#pragma interrupt_handler uart0_rx_isr:19
void uart0_rx_isr( void )
{
	unsigned char data, tmphead;

	data = UDR0; /* read the received data */
	/* calculate buffer index */
	tmphead = ( UART_RxHead + 1 ) & UART_RX_BUFFER_MASK;
	UART_RxHead = tmphead; /* store new index */
	if ( tmphead == UART_RxTail )
		{
		/* ERROR! Receive buffer overflow */
		}
	UART_RxBuf[tmphead] = data; /* store received data in buffer */
}

#pragma interrupt_handler uart0_udre_isr:20
void uart0_udre_isr( void )
{
 unsigned char tmptail;

	/* check if all data is transmitted */
	if ( UART_TxHead != UART_TxTail )
	 {
		/* calculate buffer index */
		tmptail = ( UART_TxTail + 1 ) & UART_TX_BUFFER_MASK;
		UART_TxTail = tmptail; /* store new index */
		UDR0 = UART_TxBuf[tmptail]; /* start transmition */
	 }
	else
	 {
		UCSR0B &= ~(1<<UDRIE0); /* disable UDRE interrupt */
     }
}

/* Read and write functions */
unsigned char DataInReceiveBuffer( void )
{
	return ( UART_RxHead != UART_RxTail ); 
		/* return 0 (FALSE) if the receive buffer is empty */
}

unsigned char ReceiveByte( void )
{
	unsigned char tmptail;

	while ( UART_RxHead == UART_RxTail ) /* wait for incomming data */
		;
	tmptail = ( UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;/* calculate buffer index */
	UART_RxTail = tmptail; /* store new index */
	return UART_RxBuf[tmptail]; /* return data */
}

int putchar(char c)
{
 unsigned char tmphead;

	if (c == '\n')
		putchar('\r');

	/* calculate buffer index */
	tmphead = ( UART_TxHead + 1 ) & UART_TX_BUFFER_MASK; 

	/* wait for free space in buffer */
	while ( tmphead == UART_TxTail )  ;

	UART_TxBuf[tmphead] = c; /* store data in buffer  */
	UART_TxHead = tmphead;   /* store new index       */
	UCSR0B |= (1<<UDRIE0);   /* enable UDRE interrupt */
	return c;
}

